##- Nanar <nanardon@mandrake.org>
##-
##- This program is free software; you can redistribute it and/or modify
##- it under the terms of the GNU General Public License as published by
##- the Free Software Foundation; either version 2, or (at your option)
##- any later version.
##-
##- This program is distributed in the hope that it will be useful,
##- but WITHOUT ANY WARRANTY; without even the implied warranty of
##- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##- GNU General Public License for more details.
##-
##- You should have received a copy of the GNU General Public License
##- along with this program; if not, write to the Free Software
##- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Buildmedia.pm,v 1.1 2005/04/19 23:08:32 othauvin Exp $

package Hdlist::Buildmedia;

use strict;
use warnings;
use Hdlist;
use File::Find;
use Digest::MD5;

sub new {
    my ($class, %options) = @_;

    my $Buildmedia = {
        hdlist => 'hdlist.cz',
        synthesis => 'synthesis.hdlist.cz',
        md5sum => 'MD5SUM',
        list => 'list',
        topdir => undef,
        rpmsdir => undef,
        infosdir => undef,
        recursive => undef,
        hdrsdir => undef,

        # What do we build
        buildhdrs => 0,
        buildsynthesis => 1,
        buildhdlist => 1,
        buildmd5 => 1,
        checkrpms => 0,

        force => 0,
        
        b_log => undef,
        verbosity => 4,
    };
    
    foreach (keys(%$Buildmedia)) {
        $Buildmedia->{$_} = $options{$_} if (defined($options{$_}));
    }
    
    bless($Buildmedia, $class);
    $Buildmedia->build_path();
    $Buildmedia
}

sub b_log {
    my ($self, $level, $msg, @val) = @_;
    return if ($level > $self->{verbosity});
    my @err_level = qw(ERROR WARNING INFO DEBUG);
    print STDERR $err_level[$level] .": ";
    printf(STDERR "$msg\n", @val);
}

sub build_path {
    my ($self) = @_;
    my %path;

    $path{topdir} = $self->{topdir} || $self->{rpmsdir};
    $path{infosdir} = $self->{infosdir} || $self->{rpmsdir};
    $path{hdrsdir} = $self->{hdrsdir} || "$self->{rpmsdir}/headers";
    $self->b_log(3, "topdir   is %s", $path{topdir});
    $self->b_log(3, "infosdir is %s", $path{infosdir});
    $self->b_log(3, "hdrsdir  is %s", $path{hdrsdir});

    foreach my $f (qw(hdlist synthesis list md5sum)) {
        #print "$self->{infosdir} $self->{rpmsdir} $self->{$f}\n";
        $path{$f} = ($self->{infosdir} || $self->{rpmsdir}) . '/' . $self->{$f};
        $self->b_log(3, "$f  is %s", $path{$f});
    }
    $self->{buildpath} = \%path;
}

sub findrpms {
    my ($self) = @_;
    my @rpms;
    my @time;
    $self->b_log(2, "Finding rpm in %s", $self->{rpmsdir});
    find( {
            wanted => sub {
                -d $_ and do {
                    !$self->{recursive} && $_ ne '.' and do {
                        $File::Find::prune = 1;
                        return;
                    };
                push(@time, (stat($_))[9]);    
                };
                m/\.rpm$/ and push(@rpms, $File::Find::name);
            },
            follow => 1,
            follow_skip => 2,
        },
        $self->{rpmsdir}
    );
    $self->b_log(3, "Found %d rpm in %s", scalar(@rpms), $self->{rpmsdir});

    if (!$self->{force}) {
        my ($lasttime) = sort { $b <=> $a } @time;
        -e $self->{buildpath}{hdlist} && (stat($self->{buildpath}{hdlist}))[9] > $lasttime and
            $self->{buildhdlist} = 0;
        -e $self->{buildpath}{synthesis} && (stat($self->{buildpath}{synthesis}))[9] > $lasttime and
            $self->{buildsynthesis} = 0;
    }
    
    sort(@rpms)
}

sub build {
    my ($self, %options) = @_;
    my @reqfiles;
    $self->stage1(%options,
        callback => sub {
            my (%arg) = @_;
            defined($arg{header}) or do {
                $self->b_log(1, "Can't read rpm %s", $arg{file});
                return;
            };
            push(@reqfiles, grep { m:^/: } $arg{header}->tag(1049));
        }
    );
    { my %uniq; $uniq{$_} = 1 foreach(@reqfiles); @reqfiles = keys(%uniq); }
    $self->stage2(providefiles => [@reqfiles]);
}

sub stage1 {
    my ($self, %options) = @_;
    my %existhdrs;
    
    if ($self->{buildhdrs}) {
        -d $self->{buildpath}{hdrsdir} or do {
            $self->b_log(2, "Creating headers dir %s", $self->{buildpath}{hdrsdir});
            mkdir($self->{buildpath}{hdrsdir}) or die "Can't create headers dir $self->{buildpath}{hdrsdir}: $!";
        };
    }

    -d $self->{buildpath}{infosdir} or do {
        $self->b_log(2, "Creating headers dir %s", $self->{buildpath}{infosdir});
        mkdir($self->{buildpath}{infosdir}) or die "Can't create headers dir $self->{buildpath}{infosdir}: $!";
    };

    my $h_hdlist;
    if ($self->{buildhdlist}) {
        $self->b_log(2, "Writing hdlist file %s", $self->{buildpath}{hdlist});
        open($h_hdlist, "| gzip --best > '$self->{buildpath}{hdlist}'") or die "Can't open $self->{buildpath}{hdlist}";
    }
    
    my $db = Hdlist::newdb();
    $self->b_log(3, "Reading rpms, check: %s", $self->{checkrpms} ? "Yes" : "No");
    $db->vsflags($self->{checkrpms} ? [ "NOSIGNATURES" ] : [ qw(NOSIGNATURES NOPAYLOAD NODIGESTS) ]);
    foreach my $rpm ($self->findrpms()) {
        my $header = $db->rpm2header($rpm) or do {
            $self->b_log(1, "Can't read rpm file: %s", $rpm);
            next;
        };
        
        defined($options{callback}) and $options{callback}->(header => $header);
        
        if ($self->{buildhdlist}) {
            $header->write($h_hdlist);
        }
        
        if ($self->{buildhdrs}) {
            my $hdr = $self->{buildpath}{hdrsdir} . '/' . $header->fullname .".hdr";
            $existhdrs{$hdr} = 1;
            unless (-e $hdr && (stat($rpm))[9] < (stat($hdr))[9]) {
                $self->b_log(3, "Wrinting hdr file: %s", $_);
                open(my $h_hdr, "| gzip --best > '$hdr'") or die "Can't open $hdr";
                $header->write($h_hdr, 1);
                close($h_hdr);
            }
        }
    }
    if ($self->{buildhdlist}) {
        close($h_hdlist) if ($h_hdlist);
    }

    if ($self->{buildhdrs}) {
        foreach (glob("$self->{buildpath}{hdrsdir}/*.hdr")) {
            defined($existhdrs{$_}) or do {
                $self->b_log(3, "Deleting hdr file: %s", $_);
                unlink($_);
            }
        }
    }
    $self->{current_options} = \%options; # keeping allready set options for stage2
}

sub stage2 {
    my ($self, %options) = @_;

    if ($self->{buildsynthesis}) {
        $self->b_log(3, "Wrinting synthesis %s", $self->{buildpath}{synthesis});
        open(my $h_synthesis, "| gzip --best > '$self->{buildpath}{synthesis}'") or die "Can't open $self->{buildpath}{synthesis}";
        $self->b_log(3, "Re-Reading hdlist %s", $self->{buildpath}{hdlist});
        Hdlist::parsehdlist(files => [ $self->{buildpath}{hdlist} ],
            callback => sub {
                my (%arg) = @_;
                defined($arg{header}) or return;
                $arg{header}->writesynthesis($h_synthesis, $options{providefiles});
            },
        );
        close($h_synthesis);
    }
    
    if ($self->{buildmd5}) {
        $self->b_log(3, "Writing MD5SUM %s", $self->{buildpath}{md5sum});
        open(my $h_md5, ">", $self->{buildpath}{md5sum}) or die "Can't open $self->{buildpath}{md5sum}";
        foreach my $file (qw(hdlist synthesis)) {
            my $md5 = Digest::MD5->new;
            open(my $hf, "<", $self->{buildpath}{$file}) or next;
            $md5->add($hf);
            printf($h_md5 "%s  %s\n", $md5->hexdigest, $self->{$file});
            close($hf);
        }
        close($h_md5);
    }
}

1;

__END__

=head1 NAME

Hdlist::Buildmedia - Perl module to generate mandrake repository

=head1 SYNOPSIS

use Hdlist::Buildmedia;

my $media = Hdlist::Buildmedia->new(
    rpmsdir => "/my/rpms/dir",
    recursive => 1,
);
$media->build();

=head1 DESCRIPTION

This module contains basics fonctions to generate hdlist (header collection,
uses by urpmi), synthesis (uses by urpmi) and headers files (uses by yum).

=head1 FUNCTIONS

=head1 SEE ALSO

L<rpm(8)>,

This aims at replacing part of the functionality provided by URPM.

=cut
