##- Nanar <nanardon@mandrake.org>
##-
##- This program is free software; you can redistribute it and/or modify
##- it under the terms of the GNU General Public License as published by
##- the Free Software Foundation; either version 2, or (at your option)
##- any later version.
##-
##- This program is distributed in the hope that it will be useful,
##- but WITHOUT ANY WARRANTY; without even the implied warranty of
##- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##- GNU General Public License for more details.
##-
##- You should have received a copy of the GNU General Public License
##- along with this program; if not, write to the Free Software
##- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Builddistrib.pm,v 1.1 2005/04/19 23:08:32 othauvin Exp $

package Hdlist::Buildmedia::Builddistrib;

use strict;
use warnings;
use Hdlist;
use Hdlist::Buildmedia;

sub new {
    my ($class, %options) = @_;
    my $Builddistrib = {
        distribdir => undef,
        infosdir => undef,
        
        hdlists => "hdlists",
        
        version => 'VERSION',
        md5sum => 'MD5SUM',

        recursive => 0,
        skipbadmedia => 1,
        
        distribvendor => undef,
        distribname => undef,
        distribversion => undef,
        distribtype => undef,
        distribarch => undef,

        log => sub {
            my ($m, @v) = @_;
            printf STDERR "$m\n", @v;
        },
    };

    foreach (keys(%$Builddistrib)) {
        exists($options{$_}) and $Builddistrib->{$_} = $options{$_};
    }
    
    $Builddistrib->{medias} = [];
    
    bless($Builddistrib, $class);

    return $Builddistrib->loadconfig() ? $Builddistrib : undef;
}

sub loadconfig {
    my ($self) = @_;
    my $result = 1;

    # keep compatibility with older distro
    $self->{have_media_info} = (-d "$self->{distribdir}/media/media_info" || -l _) ? 
        1 :
        0;

    $self->{infosdir} ||= ($self->{have_media_info} ? 
        "$self->{distribdir}/media/media_info" :
        "$self->{distribdir}/Mandrake/base");
 
    my ($h_hdlists, $res);
    foreach my $hdlists ($self->{hdlists}, "$self->{infosdir}/$self->{hdlists}") {
        $res = open($h_hdlists, "<", $hdlists) and last;
    }
    $res or do {
        $self->{log}->("Error: Unable to open hdlists %s", $self->{hdlists});
        return 0;
    };
    while (my $line = <$h_hdlists>) {
        chomp($line);
        my ($hdlist, $dir, $desc) = split(/\s+/,$line, 3);
        if (my $buildmedia = Hdlist::Buildmedia->new(
            topdir => "$self->{distribdir}/$dir",
            infosdir => $self->{infosdir},
            recursive => $self->{recursive},
            hdlist => $hdlist,
            synthesis => "synthesis.$hdlist",
            list => undef,
            media_name => $desc,
            media_version => $self->{distribversion},
            md5sum => undef,
            version => undef,

            'log' => $self->{log},
            
        )) {        
            push(@{$self->{medias}}, $buildmedia);
        } else {
            $self->{log}->("Error: Can't load media %s (%s)%s", $desc, $dir, $self->{skipbadmedia} ? ", skipping" : "");
            $self->{skipbadmedia} or $result = 0;
        }
    }
    close($h_hdlists);

    # Reading VERSION file to get info
    if ($self->{version} && open(my $h_version, "<", "$self->{distribdir}/$self->{version}")) {
        my @info = <$h_version> =~ m/^(.*) (.*) (.*)-(.*)-(.*) (.*) (.*)$/;
        $self->{distribvendor}  ||= $info[0];
        $self->{distribname}    ||= $info[1];
        $self->{distribversion} ||= $info[2];
        $self->{distribtype}    ||= $info[3];
        $self->{distribarch}    ||= $info[4];
        close($h_version);
    }

    $result;
}

sub build {
    my ($self, %options) = @_;
    
    my @reqfiles;
    my %group;

    if ($options{clean}) {
        foreach (@{$self->{medias}}) {
            $_->clean();
        }
    }
    
    foreach my $media (@{$self->{medias}}) {
         $media->findrpms(
            checkrpms => $options{checkrpms},
            nobadrpm => $options{nobadrpm},
            callback => sub {
                my (%arg) = @_;
                $arg{header} or return 1;
                $arg{header}->issrc() and return 1;
                $arg{header}->tag(1016) or return 1;
                push @{$group{$arg{header}->tag(1016)}}, $arg{header}->tag(1000);
                1;
            },
        ) or return 0;
        push(@reqfiles, $media->requiredfiles());
    }
    { my %uniq; $uniq{$_} = 1 foreach(@reqfiles); @reqfiles = keys(%uniq); }
    
    my $countrebuild = 0;
    foreach my $media (@{$self->{medias}}) {
        my $r = $media->build(
            filestoprovides => [@reqfiles],
            force => $options{force},
        );
        $r >= 0 or $self->{log}->("Error: Can't properly build %s", $media->medianame());
        $countrebuild += $r;
    }

    if ($self->{md5sum} && ($countrebuild || ! -e "$self->{infosdir}/$self->{md5sum}")) {
        $self->{log}->("Building md5sum %s", "$self->{infosdir}/$self->{md5sum}");
        if(open(my $h_md5sum, ">", "$self->{infosdir}/$self->{md5sum}")) {
            foreach (@{$self->{medias}}) {
                $_->genmd5($h_md5sum);
            }
            close($h_md5sum);
        } else {
            $self->{log}->("Error: Can write md5sum %s", "$self->{infosdir}/$self->{md5sum}");
        }
    }
    
    if ($self->{version} && ($countrebuild || ! -e "$self->{distribdir}/$self->{version}")) {
        $self->{log}->("Building version %s", "$self->{infosdir}/$self->{version}");
        if(open(my $h_version, ">", "$self->{distribdir}/$self->{version}")) {
            my @d = gmtime(time);
            printf $h_version "%s %s %s-%s-%s %04d%02d%02d %02d:%02d",
                $self->{distribvendor} || 'Mandrakelinux',
                $self->{distribname} || '(unknown)',
                $self->{distribversion} || '(version)',
                $self->{distribtype} || '(unknown)',
                $self->{distribarch} || '(arch)',
                $d[5] + 1900, $d[4] + 1, $d[3],
                $d[2], $d[1];
                
            close($h_version);
        } else {
            $self->{log}->("Error: Can't write version file %s", "$self->{distribdir}/$self->{version}");
        }
    }
}

1;

__END__

=head1 NAME

Hdlist::Buildmedia::Builddistrib - Perl module to generate mandrake repository

=head1 SYNOPSIS

=head1 DESCRIPTION

This module provide function to properly generate mandrake's distrib
repository.

=head1 FUNCTIONS

=item B<Hdlist::Buildmedia::Builddistrib->new(%options)>

Create a new Builddistrib object.

options are:

- distribdir: the top level of the distrib
- infosdir: index files location (default: $distrib/media/media_info)
- hdlists: the hdlists file name (default: 'hdlists')
- version: the version file name (default: 'VERSION')
- md5sum: the md5sum file name (default: 'MD5SUM')
- distribvendor: the name to put in version file (default: Mandrakelinux)
- distribname: the name of the distrib
- distribversion: the version of the distrib
- distribtype: the distrib type
- distribarch: the arch the distrib is made to
- recursive: for each media, looking for rpms recursively
- skipbadmedia: if set, failed to create object if a media is not parsable

=item B<$distrib->build(%options)>

Generate index files for the distrib.

options:
force: force index genration even rebuild is not need
checkrpms: checking rpms integrity during first pass
clean: clean index files before build
nobadrpm: stop build if a wrong rpm is found


=head1 SEE ALSO

=cut
